<?php

class Olarkchat extends Tools_Plugins_Abstract {

    const OLARKCHAT_SECURE_TOKEN = 'OlarkchatToken';

    const OLARKCHAT_WEBHOOK_ENDPOINT = 'api/olarkchat/leads/olarktoken/';

    /**
     * Toaster layout
     *
     * @var
     */
    private $_layout = null;

    public function  __construct($options, $seotoasterData)
    {
        parent::__construct($options, $seotoasterData);
        $this->_layout = new Zend_Layout();
        $this->_layout->setLayoutPath(Zend_Layout::getMvcInstance()->getLayoutPath());

        if (($scriptPaths = Zend_Layout::getMvcInstance()->getView()->getScriptPaths()) !== false) {
            $this->_view->setScriptPath($scriptPaths);
        }
        $this->_view->addScriptPath(__DIR__ . '/system/views/');
    }

    public $languagesArray = array(
        'us' => 'en-US',
        'de' => 'de-DE',
        'es' => 'es-ES',
        'fr' => 'fr-FR',
        'it' => 'it-IT',
        'pt' => 'pt-BR',
        'ru' => 'ru-RU'
    );

    /**
     * Manage config action
     */
    public function configAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            $olarkChatSettingsMapper = Olarkchat_Models_Mappers_OlarkchatSettingMapper::getInstance();
            if ($this->_request->isPost()) {
                $configParams = filter_var_array($this->_request->getParams(), FILTER_SANITIZE_STRING);
                $identify = $configParams['identify'];
                unset($configParams['webhook_id']);

                $secureToken = $this->_request->getParam(Tools_System_Tools::CSRF_SECURE_TOKEN, false);
                $tokenValid = Tools_System_Tools::validateToken($secureToken, self::OLARKCHAT_SECURE_TOKEN);
                if (!$tokenValid) {
                    $this->_responseHelper->fail('');
                }
                $olarkChatSettings = $olarkChatSettingsMapper->getConfigParams();

                if (empty($identify)) {
                    $configParams['identify'] = '';
                    if (isset($olarkChatSettings['identify'])) {
                        $configParams['identify'] = $olarkChatSettings['identify'];
                    }
                }

                if (empty($identify)) {
                    $this->_responseHelper->fail($this->_translator->translate('Olark live chat not configured!'));
                }

                $olarkChatSettingsMapper->save($configParams);

                $this->_responseHelper->success('');
            } else {
                $olarkChatSettings = $olarkChatSettingsMapper->getConfigParams();
                if (empty($olarkChatSettings['webhook_id'])) {
                    $olarkChatSettingsMapper->save(array('webhook_id' => sha1(time().rand())));
                    $olarkChatSettings = $olarkChatSettingsMapper->getConfigParams();
                }
                $this->_view->settings = $olarkChatSettings;
                $this->_view->translator = $this->_translator;
                $this->_layout->content = $this->_view->render('olarkChatConfig.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function rulesConfigAction()
    {
        if (Tools_Security_Acl::isAllowed(Tools_Security_Acl::RESOURCE_PLUGINS)) {
            if ($this->_request->isGet()) {
                $this->_layout->content = $this->_view->render('olarkChatRulesConfig.phtml');
                echo $this->_layout->render();
            }
        }
    }

    public function _makeOptionChat()
    {
        $olarkChatSettingsMapper = Olarkchat_Models_Mappers_OlarkchatSettingMapper::getInstance();
        $olarkChatSettings = $olarkChatSettingsMapper->getConfigParams();

        $configHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('config');
        $systemLanguage = $configHelper->getConfig('language');
        if(array_key_exists($systemLanguage, $this->languagesArray)){
            $language = $this->languagesArray[$systemLanguage];
        }else{
            $language = $this->languagesArray['us'];
        }
        $this->_view->language = $language;

        $this->_view->identify = $olarkChatSettings['identify'];
        $this->_view->enableOlark = $olarkChatSettings['enableOlark'];//Enable/Disable Chat
        $this->_view->showOnMobile = $olarkChatSettings['showOnMobile'];//Hide chatbox on mobile
        $this->_view->startChatBox = $olarkChatSettings['startChatBox'];//Expand the chatbox
        $this->_view->detachChatbox = $olarkChatSettings['detachChatbox'];//Floating Chatbox
        $this->_view->showAsTab = $olarkChatSettings['showAsTab'];//Show as Tab
        $this->_view->changeChatboxPosition = $olarkChatSettings['changeChatboxPosition'];//Change Chatbox Position
        $this->_view->setDarkTheme = $olarkChatSettings['setDarkTheme'];//Set Dark Theme
        $this->_view->changeChatboxSize = $olarkChatSettings['changeChatboxSize'];//Change Chatbox Size
        $this->_view->setThemeColor = $olarkChatSettings['setThemeColor'];//Set Theme Color
        $this->_view->changeColorVisitorMessages = $olarkChatSettings['changeColorVisitorMessages'];//Change the Color of Visitor Messages

        return $this->_view->render('olarkLiveChat.phtml');
    }

}