<?php

class Api_Olarkchat_Leads extends Api_Service_Abstract
{

    /**
     * System Olark inbound type
     */
    const INBOUND_CHAT = 'inbound_chat';

    /**
     * System response helper
     *
     * @var null
     */
    protected $_responseHelper = null;

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array('olarktoken');


    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('post')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('post')
        ),
        Tools_Security_Acl::ROLE_GUEST => array(
            'allow' => array('post')
        )
    );

    /*
     * Default timezone format
     */
    public $defaultTimezone = 'UTC';

    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/olarkchat/leads/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
    }

    /**
     * Create new lead
     *
     * Resource:
     * : /api/olarkchat/leads/
     *
     * HttpMethod:
     * : POST
     *
     * @return JSON
     */
    public function postAction()
    {
        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error($translator->translate('Missing mandatory params'));
        }

        $olarkChatSettingsMapper = Olarkchat_Models_Mappers_OlarkchatSettingMapper::getInstance();
        $webHookId = $olarkChatSettingsMapper->getConfigParam('webhook_id');
        if (empty($webHookId) || empty($data['olarktoken']) || $data['olarktoken'] !== $webHookId) {
            $this->_error($translator->translate('Wrong token provided'));
        }

        if (empty($data['data'])) {
            $this->_error($translator->translate('Empty params'));
        }

        $params = json_decode(html_entity_decode($data['data'], ENT_COMPAT | ENT_HTML401, 'UTF-8'), true);
        if (empty($params)) {
            $this->_error($translator->translate('Wrong params'));
        }

        $processLeadData = $this->_prepareLeadsData($params);
        if ($processLeadData['error'] === '0') {
            $processLeadData['leadsData']['specialInboundType'] = self::INBOUND_CHAT;

            $leadProcessingResponse = Leads::processLead($processLeadData['leadsData']);
            if ($leadProcessingResponse['error'] == '1') {
                $this->_error($translator->translate($leadProcessingResponse['message']));
            } else {
                $this->_responseHelper->success($translator->translate('Transcript has been processed'));
            }
        } else {
            $this->_error($translator->translate($processLeadData['message']));
        }

    }

    /**
     *
     * Resource:
     * : /api/olarkchat/leads/
     *
     * HttpMethod:
     * : PUT
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function putAction()
    {

    }

    /**
     *
     * Resource:
     * : /api/olarkchat/leads/
     *
     * HttpMethod:
     * : DELETE
     *
     * ## Parameters:
     * id (type integer)
     *
     * @return JSON
     */
    public function deleteAction()
    {

    }

    /**
     * Prepare lead data
     *
     * @param array $data raw olark transcript data
     * @return array
     */
    private function _prepareLeadsData($data)
    {
        $visitor = $data['visitor'];
        $leadsData = array();
        $leadsData['eventName'] = 'olark';
        $currentTime = Tools_System_Tools::convertDateFromTimezone('now', $this->defaultTimezone, Tools_System_Tools::getUserTimezone(), 'H:i:s');
        $formDate = Tools_System_Tools::convertDateFromTimezone('now', $this->defaultTimezone, Tools_System_Tools::getUserTimezone(), 'Y-m-d ') . $currentTime;
        $leadsData['eventDate']        = $formDate;
        $leadsData['lead_email'] = $visitor['emailAddress'];
        $fullNameAttrubutes = explode(' ', $visitor['fullName']);
        $leadsData['lead_first_name'] = $fullNameAttrubutes['0'];
        if (!empty($fullNameAttrubutes['1'])) {
            $leadsData['lead_last_name'] = $fullNameAttrubutes['1'];
        } else {
            $leadsData['lead_last_name'] = '';
        }

        $translator = Zend_Registry::get('Zend_Translate');
        $eventName = $translator->translate('Olark chat');

        $leadsData['lead_city'] = $visitor['city'];
        $leadsData['lead_mobile'] = $visitor['phoneNumber'];
        $leadsData['lead_phone_country_code'] = $visitor['countryCode'];
        $leadsData['lead_mobile_country_code'] = $visitor['countryCode'];
        $leadsData['lead_country_code'] = $visitor['countryCode'];
        if (!empty($visitor['region'])) {
            $leadsData['lead_state_code'] = $visitor['region'];
        }
        $leadsData['lead_tags'] = $data['tags'];
        $leadsData['organization_name'] = $visitor['organization'];
        $leadsData['rawData'] = $visitor;
        $leadsData['rawData']['tags'] = $data['tags'];

        //visitor_ip_address
        if(!empty($visitor['ip'])) {
            $leadsData['lead_ip_address'] = $visitor['ip'];
        }

        $attributes = Tools_OlarkchatRulesTools::qualifyRule($data);
        if (!empty($attributes)) {
            foreach ($attributes as $attrName => $attributeVal) {
                $leadsData[$attrName] = $attributeVal;
            }
        }

        $conversationAndData = Tools_LeadTools::prepareFormDetailsHtml($leadsData['rawData']). '<br/>';
        if (!empty($data['items'])) {
            foreach ($data['items'] as $item) {
                if ($item['kind'] === 'MessageToVisitor') {
                    $conversationAndData .= $item['nickname'] . ': ' . $item['body'] . '<br/>';
                } elseif ($item['kind'] === 'MessageToOperator') {
                    $conversationAndData .= $item['visitor_nickname'] . ': ' . $item['body'] . '<br/>';
                } elseif($item['kind'] === 'OfflineMessage') {
                    $conversationAndData .= trim(preg_replace('/\s+/', ' ', $item['body'])) . '<br/>';
                } else {
                    $conversationAndData .= '';
                }
            }
        }

        $leadsData['eventDescription'] = '{"shortDescription":"'.$eventName.'", "fullDescription":"'.$conversationAndData.'"}';

        $isNewLead = Tools_LeadImportTools::isNewLead($leadsData);
        if($isNewLead['status'] == Tools_LeadFormEventsTools::LEAD_MODEL_STATUS_NEW && !empty($visitor['referrer'])) {
            $leadsData['lead_source'] = $visitor['referrer'];
        }

        return array('error' => '0', 'leadsData' => $leadsData);


    }

}
