<?php

/**
 * Class Api_Olarkchat_Rules
 */
class Api_Olarkchat_Rules extends Api_Service_Abstract
{


    /**
     * Lead secure token
     */
    const OLARKCHAT_SECURE_TOKEN = 'OlarkchatToken';

    /**
     * Mandatory fields
     *
     * @var array
     */
    protected $_mandatoryParams = array();

    /**
     * @var array Access Control List
     */
    protected $_accessList = array(
        Tools_Security_Acl::ROLE_SUPERADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        ),
        Tools_Security_Acl::ROLE_ADMIN => array(
            'allow' => array('get', 'post', 'put', 'delete')
        )
    );


    public function init()
    {
        parent::init();
        $this->_responseHelper = Zend_Controller_Action_HelperBroker::getStaticHelper('response');
    }


    /**
     *
     * Resource:
     * : /api/olarkchat/rules/
     *
     * HttpMethod:
     * : GET
     *
     * @return JSON
     */
    public function getAction()
    {
        $limit = filter_var($this->_request->getParam('limit', null), FILTER_SANITIZE_NUMBER_INT);
        $offset = filter_var($this->_request->getParam('offset', null), FILTER_SANITIZE_NUMBER_INT);
        $order = filter_var($this->_request->getParam('order', 'plorgc.created_at DESC'), FILTER_SANITIZE_STRING);
        $id = filter_var($this->_request->getParam('id', null), FILTER_SANITIZE_NUMBER_INT);

        $where = null;
        $withoutCount = false;
        $singleRecord = false;
        $olarkchatRulesGeneralConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper::getInstance();
        if (!empty($id)) {
            $where = $olarkchatRulesGeneralConfigMapper->getDbTable()->getAdapter()->quoteInto('plorgc.id = ?', $id);
            $withoutCount = true;
            $singleRecord = true;
        }

        $olarkChatData = $olarkchatRulesGeneralConfigMapper->fetchAll($where, $order, $limit, $offset, $withoutCount,
            $singleRecord);

        if (!empty($id)) {
            $olarkchatRulesActionsMapper = Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper::getInstance();
            $actionsData = $olarkchatRulesActionsMapper->getActionByRuleIds(array($id));
            $olarkChatData['actionsData'] = $actionsData;
            $olarkchatRulesConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesConfigMapper::getInstance();
            $fieldsData = $olarkchatRulesConfigMapper->getFieldsByRuleId($id);
            $olarkChatData['fieldsData'] = $fieldsData;

        }

        $owners = Leads_Mapper_LeadsMapper::getInstance()->getLeadOwnersFullList();

        $olarkChatData['configData']['leadOwners'] = $owners;

        return array('olarkChatData' => $olarkChatData, 'status' => 'ok');
    }

    /**
     *
     * * * Resource:
     * : /api/olarkchat/rules/
     *
     * @return array
     */
    public function postAction()
    {
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::OLARKCHAT_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array('status' => 'error', 'message' => 'Access denied', 'errorCode' => '1'), 200);
        }

        $data = filter_var_array($this->getRequest()->getParams(), FILTER_SANITIZE_STRING);
        $translator = Zend_Registry::get('Zend_Translate');

        $fieldDataMissing = array_filter($this->_mandatoryParams, function ($param) use ($data) {
            if (!array_key_exists($param, $data) || empty($data[$param])) {
                return $param;
            }
        });

        if (!empty($fieldDataMissing)) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate('Missing mandatory params'),
                'errorCode' => '1'
            ), 200);
        }

        if (empty($data['fieldsData']) || empty($data['actionsData'])) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate('Missing fields and actions data'),
                'errorCode' => '1'
            ), 200);
        }

        foreach ($data['fieldsData'] as $key => $fieldData) {
            if ($fieldData['name'] === 'domain') {
                $fullDomain = 'http://' . strtr($fieldData['value'], array('http://' => '', 'https://' => ''));
                $domain = parse_url($fullDomain, PHP_URL_HOST) . parse_url($fullDomain, PHP_URL_PATH);
                if (empty($domain)) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate('not valid domain name'),
                        'errorCode' => '1'
                    ), 200);
                }
                $domain  = rtrim($domain, '/');
                $domain = $domain.'/';
                $domain = substr($domain, 0, strrpos($domain, '/'));
                $data['fieldsData'][$key]['value'] = $domain;
            }

            if ($fieldData['name'] === 'url') {
                $url = parse_url($fieldData['value']);
                if (empty($url)) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate('not valid url'),
                        'errorCode' => '1'
                    ), 200);
                }
                $data['fieldsData'][$key]['value'] = preg_replace('~^(.*[\\\/])~', '',$url['path']);
            }
        }


        $olarkchatRulesGeneralConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper::getInstance();
        $olarkchatRulesConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesConfigMapper::getInstance();
        $olarkchatRulesActionsMapper = Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper::getInstance();

        $ruleName = $data['rule_name'];

        $ruleModel = $olarkchatRulesGeneralConfigMapper->checkRuleExist($ruleName);
        if ($ruleModel instanceof Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate('Rule name already exists'),
                'errorCode' => '1'
            ), 200);
        }

        $ruleModel = new Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel();

        $createdAt = Tools_System_Tools::convertDateFromTimezone('now');
        if (empty($data['user_id'])) {
            $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
        } else {
            $currentUserId = $data['user_id'];
        }

        $ruleModel->setOptions($data);
        $ruleModel->setCreatorId($currentUserId);
        $ruleModel->setCreatedAt($createdAt);

        $ruleModel = $olarkchatRulesGeneralConfigMapper->save($ruleModel);
        $ruleId = $ruleModel->getId();

        if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
            foreach ($data['fieldsData'] as $fieldData) {
                if (empty($fieldData['name'])) {
                    continue;
                }
                $olarkchatRulesConfigModel = new Olarkchat_Models_Models_OlarkchatRulesConfigModel();
                $olarkchatRulesConfigModel->setFieldName($fieldData['name']);
                if (is_array($fieldData['value'])) {
                    $fieldValue = implode(',', array_map('trim', $fieldData['value']));
                } else {
                    $fieldValue = implode(',', array_map('trim', explode(',', $fieldData['value'])));
                }
                $olarkchatRulesConfigModel->setFieldValue($fieldValue);
                $olarkchatRulesConfigModel->setRuleId($ruleId);
                if (!empty($fieldData['operator'])) {
                    $olarkchatRulesConfigModel->setRuleComparisonOperator($fieldData['operator']);
                }
                $olarkchatRulesConfigMapper->save($olarkchatRulesConfigModel);
            }
        }

        if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
            foreach ($data['actionsData'] as $actionData) {
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                    $actionData['creator_id'] = $currentUserId;
                }
                if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                    $actionData['user_id'] = $currentUserId;
                }
                if ($actionData['actionType'] === 'assign_email_copy') {
                    $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                }

                if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                    $shoppingConfigMapper = Models_Mapper_ShoppingConfig::getInstance();
                    $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                    if (!empty($actionData['taskTimezone'])) {
                        $timezone = $actionData['taskTimezone'];
                    } elseif (!empty($storeTimezone)) {
                        $timezone = $storeTimezone;
                    } else {
                        $timezone = 'UTC';
                    }
                    $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                    if ($hoursShift > 0) {
                        $hoursShift = '-' . $hoursShift;
                    } else {
                        $hoursShift = '+' . abs($hoursShift);
                    }
                    $actionData['hoursShift'] = $hoursShift;
                }
                $olarkchatRulesActionModel = new Olarkchat_Models_Models_OlarkchatRulesActionModel();
                $olarkchatRulesActionModel->setRuleId($ruleId);
                $olarkchatRulesActionModel->setActionType($actionData['actionType']);
                unset($actionData['actionType']);
                $olarkchatRulesActionModel->setActionConfig(json_encode($actionData));
                $olarkchatRulesActionsMapper->save($olarkchatRulesActionModel);
            }

            return array('status' => 'ok', 'message' => $translator->translate('Rule has been added'));
        }

        $this->_error(array('status' => 'error', 'message' => '', 'errorCode' => '1'), 200);
    }

    /**
     *
     * * * Resource:
     * : /api/olarkchat/rules/
     *
     *
     */
    public function putAction()
    {
        $data = json_decode($this->_request->getRawBody(), true);
        $translator = Zend_Registry::get('Zend_Translate');

        $tokenValid = Tools_System_Tools::validateToken($data['secureToken'], self::OLARKCHAT_SECURE_TOKEN);
        if ($tokenValid !== true) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate('Access denied'),
                'errorCode' => '1'
            ), 200);
        }

        $id = filter_var($data['id'], FILTER_SANITIZE_NUMBER_INT);

        foreach ($data['fieldsData'] as $key => $fieldData) {
            if ($fieldData['name'] === 'domain') {
                $fullDomain = 'http://' . strtr($fieldData['value'], array('http://' => '', 'https://' => ''));
                $domain = parse_url($fullDomain, PHP_URL_HOST) . parse_url($fullDomain, PHP_URL_PATH);
                if (empty($domain)) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate('not valid domain name'),
                        'errorCode' => '1'
                    ), 200);
                }
                $domain  = rtrim($domain, '/');
                $domain = $domain.'/';
                $domain = substr($domain, 0, strrpos($domain, '/'));
                $data['fieldsData'][$key]['value'] = $domain;
            }

            if ($fieldData['name'] === 'url') {
                $url = parse_url($fieldData['value']);
                if (empty($url)) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => $translator->translate('not valid url'),
                        'errorCode' => '1'
                    ), 200);
                }
                $data['fieldsData'][$key]['value'] = preg_replace('~^(.*[\\\/])~', '',$url['path']);
            }
        }

        if (!empty($id)) {
            if (empty($data['rule_name'])) {
                $this->_error(array('status' => 'error', 'message' => 'Rule name is missing', 'errorCode' => '1'), 200);
            }

            $olarkchatRulesGeneralConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper::getInstance();
            $olarkchatRulesConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesConfigMapper::getInstance();
            $olarkchatRulesActionsMapper = Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper::getInstance();
            $olarkchatRulesGeneralConfigModel = $olarkchatRulesGeneralConfigMapper->find($id);
            if (!$olarkchatRulesGeneralConfigModel instanceof Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel) {
                $this->_error(array('status' => 'error', 'message' => 'Rule not found', 'errorCode' => '1'), 200);
            }

            if ($olarkchatRulesGeneralConfigModel->getRuleName() != $data['rule_name']) {
                $anotherRuleWithSameName = $olarkchatRulesGeneralConfigMapper->checkRuleExist($data['rule_name']);
                if ($anotherRuleWithSameName instanceof Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel) {
                    $this->_error(array(
                        'status' => 'error',
                        'message' => 'Another rule has the same name',
                        'errorCode' => '1'
                    ), 200);
                }
            }



            if (empty($data['user_id'])) {
                $currentUserId = Zend_Controller_Action_HelperBroker::getStaticHelper('session')->getCurrentUser()->getId();
            } else {
                $currentUserId = $data['user_id'];
            }

            $olarkchatRulesGeneralConfigModel->setEditorId($currentUserId);
            $olarkchatRulesGeneralConfigModel->setUpdatedAt(Tools_System_Tools::convertDateFromTimezone('now'));
            $olarkchatRulesGeneralConfigModel->setRuleName($data['rule_name']);
            $olarkchatRulesGeneralConfigMapper->save($olarkchatRulesGeneralConfigModel);


            $olarkchatRulesConfigMapper->deleteByRuleId($id);
            $olarkchatRulesActionsMapper->deleteByRuleId($id);

            if (!empty($data['fieldsData']) && is_array($data['fieldsData'])) {
                foreach ($data['fieldsData'] as $fieldData) {
                    if (empty($fieldData['name'])) {
                        continue;
                    }
                    $olarkchatRulesConfigModel = new Olarkchat_Models_Models_OlarkchatRulesConfigModel();
                    $olarkchatRulesConfigModel->setFieldName($fieldData['name']);
                    if (is_array($fieldData['value'])) {
                        $fieldValue = implode(',', array_map('trim', $fieldData['value']));
                    } else {
                        $fieldValue = implode(',', array_map('trim', explode(',', $fieldData['value'])));
                    }
                    $olarkchatRulesConfigModel->setFieldValue($fieldValue);
                    $olarkchatRulesConfigModel->setRuleId($id);
                    if (!empty($fieldData['operator'])) {
                        $olarkchatRulesConfigModel->setRuleComparisonOperator($fieldData['operator']);
                    }
                    $olarkchatRulesConfigMapper->save($olarkchatRulesConfigModel);
                }
            }

            if (!empty($data['actionsData']) && is_array($data['actionsData'])) {
                foreach ($data['actionsData'] as $actionData) {
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['creator_id'])) {
                        $actionData['creator_id'] = $currentUserId;
                    }
                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['user_id'])) {
                        $actionData['user_id'] = $currentUserId;
                    }
                    if ($actionData['actionType'] === 'assign_email_copy') {
                        $actionData['emailsToCopy'] = Tools_LeadTools::trimValues($actionData['emailsToCopy'], true);
                    }

                    if ($actionData['actionType'] === 'assign_task' && empty($actionData['taskImmediate'])) {
                        $shoppingConfigMapper = Models_Mapper_ShoppingConfig::getInstance();
                        $storeTimezone = $shoppingConfigMapper->getConfigParam('timezone');
                        if (!empty($actionData['taskTimezone'])) {
                            $timezone = $actionData['taskTimezone'];
                        } elseif (!empty($storeTimezone)) {
                            $timezone = $storeTimezone;
                        } else {
                            $timezone = 'UTC';
                        }
                        $hoursShift = Tools_EmailSequenceTools::getTimezoneShift('UTC', $timezone);
                        if ($hoursShift > 0) {
                            $hoursShift = '-' . $hoursShift;
                        } else {
                            $hoursShift = '+' . abs($hoursShift);
                        }
                        $actionData['hoursShift'] = $hoursShift;
                    }
                    $olarkchatRulesActionModel = new Olarkchat_Models_Models_OlarkchatRulesActionModel();
                    $olarkchatRulesActionModel->setRuleId($id);
                    $olarkchatRulesActionModel->setActionType($actionData['actionType']);
                    unset($actionData['actionType']);
                    $olarkchatRulesActionModel->setActionConfig(json_encode($actionData));
                    $olarkchatRulesActionsMapper->save($olarkchatRulesActionModel);
                }
            }

            return array('error' => '0', '');

        }

        $this->_error(array('status' => 'error', 'message' => 'Missing rule id', 'errorCode' => '1'), 200);

    }

    /**
     * Delete record
     *
     * * Resource:
     * : /api/olarkchat/rules/
     *
     * @return array
     */
    public function deleteAction()
    {
        $secureToken = $this->_request->getParam('secureToken', false);
        $tokenValid = Tools_System_Tools::validateToken($secureToken, self::OLARKCHAT_SECURE_TOKEN);
        $translator = Zend_Registry::get('Zend_Translate');

        if ($tokenValid !== true) {
            $this->_error(array(
                'status' => 'error',
                'message' => $translator->translate('Access denied'),
                'errorCode' => '1'
            ), 200);
        }

        $id = filter_var($this->_request->getParam('id'), FILTER_SANITIZE_NUMBER_INT);

        $olarkchatRulesGeneralConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper::getInstance();
        $larkchatRulesGeneralConfigModel = $olarkchatRulesGeneralConfigMapper->find($id);
        if (!$larkchatRulesGeneralConfigModel instanceof Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel) {
            $this->_error(array('status' => 'error', 'message' => 'Can\'t delete record', 'errorCode' => '1'), 200);
        }
        $olarkchatRulesGeneralConfigMapper->delete($id);

        return array('status' => 'ok', 'message' => $translator->translate('Rule has been deleted'));
    }

}
