<?php

/**
 * OlarkchatRulesActionsMapper.php
 *
 *
 * @method Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Olarkchat_Models_Models_OlarkchatRulesActionModel';

    protected $_dbTable = 'Olarkchat_Models_Dbtables_OlarkchatRulesActionsDbTable';

    /**
     * Save event actions model to DB
     * @param $model Olarkchat_Models_Models_OlarkchatRulesActionModel
     * @return Olarkchat_Models_Models_OlarkchatRulesActionModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_id' => $model->getRuleId(),
            'action_type' => $model->getActionType(),
            'action_config' => $model->getActionConfig()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }


    /**
     * Get action by rule id
     *
     * @param array $ruleId rule id
     * @return array
     */
    public function getActionByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->_findWhere($where);
    }


    /**
     * Get action by rule ids
     *
     * @param array $ruleIds rule ids
     * @return array
     */
    public function getActionByRuleIds($ruleIds)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id IN (?)', $ruleIds);

        $select = $this->getDbTable()->getAdapter()->select()->from('plugin_olarkchat_rules_actions')->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Delete by rule id
     *
     * @param int $ruleId rule id
     * @return int
     */
    public function deleteByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_id = ?', $ruleId);

        return $this->getDbTable()->getAdapter()->delete('plugin_olarkchat_rules_actions', $where);
    }


    /**
     * Delete event rules actions config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_olarkchat_rules_actions', $where);

    }

}
