<?php

/**
 * OlarkchatRulesGeneralConfigMapper.php
 *
 *
 * @method Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper  getInstance()   getInstance()   Returns an instance of itself
 * @method Zend_Db_Table    getDbTable()    getDbTable()    Returns an instance of DbTable
 */
class Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper extends Application_Model_Mappers_Abstract
{

    protected $_model = 'Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel';

    protected $_dbTable = 'Olarkchat_Models_Dbtables_OlarkchatRulesGeneralConfigDbTable';

    /**
     * Save lead event general config model to DB
     * @param $model Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel
     * @return Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel
     */
    public function save($model)
    {
        if (!$model instanceof $this->_model) {
            $model = new $this->_model($model);
        }

        $data = array(
            'rule_name' => $model->getRuleName(),
            'created_at' => $model->getCreatedAt(),
            'creator_id' => $model->getCreatorId(),
            'updated_at' => $model->getUpdatedAt(),
            'editor_id' => $model->getEditorId()
        );

        $id = $model->getId();
        if (!empty($id)) {
            $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);
            $this->getDbTable()->update($data, $where);
        } else {
            $id = $this->getDbTable()->insert($data);
            $model->setId($id);
        }

        return $model;
    }

    /**
     *
     * @param string $where SQL where clause
     * @param string $order OPTIONAL An SQL ORDER clause.
     * @param int $limit OPTIONAL An SQL LIMIT count.
     * @param int $offset OPTIONAL An SQL LIMIT offset.
     * @param bool $withoutCount flag to get with or without records quantity
     * @param bool $singleRecord flag fetch single record
     * @param bool $withTimezoneOffset select data with processed dates based on logged user timezone
     * @return array
     */
    public function fetchAll(
        $where = null,
        $order = null,
        $limit = null,
        $offset = null,
        $withoutCount = false,
        $singleRecord = false,
        $withTimezoneOffset = false
    ) {

        $mysqlOffset = Tools_System_Tools::getUtcOffset('P');

        $createdAt = 'plorgc.created_at';

        if ($withTimezoneOffset === true) {
            $createdAt = new Zend_Db_Expr("CONVERT_TZ(`plorgc`.`created_at`, '+00:00', '$mysqlOffset')");
        }

        $updatedAt = 'plorgc.updated_at';

        if ($withTimezoneOffset === true) {
            $updatedAt = new Zend_Db_Expr("CONVERT_TZ(`plorgc`.`updated_at`, '+00:00', '$mysqlOffset')");
        }

        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('plorgc' => 'plugin_olarkchat_rules_general_config'),
                array(
                    'plorgc.id',
                    'plorgc.rule_name',
                    'plorgc.creator_id',
                    'createdAt' => $createdAt,
                    'plorgc.editor_id',
                    'updatedAt' => $updatedAt,
                    'creatorName' => 'u.full_name',
                    'editorName' => 'uc.full_name',
                    'actionTypes' => new Zend_Db_Expr('GROUP_CONCAT(DISTINCT(plora.action_type))')
                )
            )
            ->joinLeft(array('plora' => 'plugin_olarkchat_rules_actions'), 'plora.rule_id=plorgc.id', array())
            ->joinLeft(array('u' => 'user'), 'u.id=plorgc.creator_id', array())
            ->joinLeft(array('uc' => 'user'), 'uc.id=plorgc.editor_id', array());


        $select->group('plorgc.id');
        if (!empty($order)) {
            $select->order($order);
        }

        if (!empty($where)) {
            $select->where($where);
        }

        $select->limit($limit, $offset);

        if ($singleRecord) {
            $data = $this->getDbTable()->getAdapter()->fetchRow($select);
        } else {
            $data = $this->getDbTable()->getAdapter()->fetchAll($select);
        }

        if ($withoutCount === false) {
            $select->reset(Zend_Db_Select::COLUMNS);
            $select->reset(Zend_Db_Select::FROM);
            $select->reset(Zend_Db_Select::LIMIT_OFFSET);
            $select->reset(Zend_Db_Select::LIMIT_COUNT);

            $count = array('count' => new Zend_Db_Expr('COUNT(DISTINCT(plorgc.id))'));

            $select->from(array('plorgc' => 'plugin_olarkchat_rules_general_config'), $count)
                ->joinLeft(array('plora' => 'plugin_olarkchat_rules_actions'), 'plora.rule_id=plorgc.id', array())
                ->joinLeft(array('u' => 'user'), 'u.id=plorgc.creator_id', array())
                ->joinLeft(array('uc' => 'user'), 'uc.id=plorgc.editor_id', array());


            $select =  $this->getDbTable()->getAdapter()->select()
                ->from(
                    array('subres' => $select),
                    array('count' => 'SUM(count)')
                );

            $count = $this->getDbTable()->getAdapter()->fetchRow($select);

            return array(
                'totalRecords' => $count['count'],
                'data' => $data,
                'offset' => $offset,
                'limit' => $limit
            );
        } else {
            return $data;
        }
    }

    /**
     * Get all config data
     *
     * @return array
     */
    public function getRulesConfigData()
    {
        $select = $this->getDbTable()->getAdapter()->select()
            ->from(array('porgc' => 'plugin_olarkchat_rules_general_config'),
                array(
                    'porgc.id',
                    'porgc.rule_name',
                    'porc.field_name',
                    'porc.rule_comparison_operator',
                    'porc.field_value'
                )
            )
            ->joinLeft(array('porc' => 'plugin_olarkchat_rules_config'), 'porgc.id=porc.rule_id', array());

        $select->order('porgc.id');

        return $this->getDbTable()->getAdapter()->fetchAll($select);
    }

    /**
     * get rules by rule id
     * @param int $ruleId rule id
     * @return array
     */
    public function getFieldsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plorc.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plorc' => 'plugin_olarkchat_rules_config'),
            array('plorc.field_name', 'plorc.id', 'plorc.rule_comparison_operator', 'plorc.field_value'))
            ->where($where)->order('plorc.field_name');

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * get actions by rule id
     * @param int $ruleId rule id
     * @return array
     */
    public function getActionsByRuleId($ruleId)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('plora.rule_id = ?', $ruleId);

        $select = $this->getDbTable()->getAdapter()->select()->from(array('plora' => 'plugin_olarkchat_rules_actions'),
            array('plora.action_type', 'plora.id', 'plora.action_config'))
            ->where($where);

        return $this->getDbTable()->getAdapter()->fetchAssoc($select);
    }

    /**
     * Check if rule already exists
     *
     * @param string $ruleName unique rule name
     * @return Olarkchat_Models_Models_OlarkchatRulesGeneralConfigModel
     */
    public function checkRuleExist($ruleName)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('rule_name = ?', $ruleName);

        return $this->_findWhere($where);
    }


    /**
     * Delete form rules general config record
     *
     * @param int $id config id
     * @return mixed
     * @throws Exception
     */
    public function delete($id)
    {
        $where = $this->getDbTable()->getAdapter()->quoteInto('id = ?', $id);

        return $this->getDbTable()->getAdapter()->delete('plugin_olarkchat_rules_general_config', $where);

    }

}
