<?php

/**
 * Class Tools_OlarkchatRulesTools
 */
class Tools_OlarkchatRulesTools
{

    /**
     * Get comparison operators legends
     *
     * @return array
     * @throws Zend_Exception
     */
    public static function getComparisonOperatorLegend()
    {
        $translator = Zend_Registry::get('Zend_Translate');

        $comparisonOperatorLegend = array(
            'equal' => $translator->translate('is'),
            'notequal' => $translator->translate('other than'),
            'like' => $translator->translate('includes'),
            'in' => $translator->translate('one of'),
            'notin' => $translator->translate('nothing of'),
            'greaterthan' => $translator->translate('greater than'),
            'lessthan' => $translator->translate('less than')
        );

        return $comparisonOperatorLegend;
    }

    /**
     * Prepare  rules
     *
     * @param array $rules rules
     * @return array
     */
    public static function prepareRules($rules)
    {
        $resultRules = array();
        foreach ($rules as $rule) {
            $resultRules[$rule['id']][] = $rule;
        }

        return $resultRules;
    }

    /**
     * Qualify rule
     *
     * @param $data
     */
    public static function qualifyRule($data)
    {
        $olarkchatRulesGeneralConfigMapper = Olarkchat_Models_Mappers_OlarkchatRulesGeneralConfigMapper::getInstance();
        $rulesData = $olarkchatRulesGeneralConfigMapper->getRulesConfigData();
        $rulesData = self::prepareRules($rulesData);
        $attributesResult = array();
        foreach ($rulesData as $ruleData) {
            $result = self::verifyRules($data, $ruleData);
            if (empty($result['error'])) {
                $attributes = self::getAttribute($result['ruleId']);
                if (!empty($attributes)) {
                    foreach ($attributes as $attrName => $attributeVal) {
                        $attributesResult[$attrName] = $attributeVal;
                    }
                }
            }
        }

        return $attributesResult;

    }

    /**
     * Get attribute for the olark chat lead data
     *
     * @param int $ruleId rule id
     * @return mixed
     */
    public static function getAttribute($ruleId)
    {
        $olarkchatRulesActionsMapper = Olarkchat_Models_Mappers_OlarkchatRulesActionsMapper::getInstance();
        $actions = $olarkchatRulesActionsMapper->getActionByRuleIds($ruleId);
        if (!empty($actions)) {
            if (!empty($actions)) {
                foreach ($actions as $action) {
                    $actionType = $action['action_type'];
                    $actionConfig = $action['action_config'];
                    if (!empty($actionConfig)) {
                        $actionConfig = json_decode($actionConfig, true);
                        if ($actionType === Olarkchat_Models_Models_OlarkchatRulesActionModel::ACTION_TYPE_ASSIGN_OWNER) {
                            return array("lead_owner_id" => $actionConfig['assign_to']);
                        }
                    }
                }
            }
        }
    }


    /**
     * Verify workflow rule condition
     *
     * @param array $data lead data
     * @param array $rules workflow score rules
     * @return array
     */
    public static function verifyRules($data, $rules)
    {
        $add = false;
        foreach ($rules as $rule) {
            $fieldName = $rule['field_name'];
            $ruleId = $rule['id'];
            $fieldValue = $rule['field_value'];
            $operator = $rule['rule_comparison_operator'];

            if ($fieldName === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_FIELD_NAME_DOMAIN) {
                $conversationPage = $data['visitor']['conversationBeginPage'];
                $domain = self::filterDomain($conversationPage);
                if (empty($domain)) {
                    $add = false;
                    continue;
                }
                $add = self::compareValue($operator, $domain, $fieldValue);

            }

            if ($fieldName === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_FIELD_NAME_URL) {
                $conversationPage = $data['visitor']['conversationBeginPage'];
                $url = preg_replace('~^(.*[\\\/])~', '', parse_url($conversationPage, PHP_URL_HOST) . parse_url($conversationPage, PHP_URL_PATH));
                if (empty($url)) {
                    $add = false;
                    continue;
                }
                $add = self::compareValue($operator, $url, $fieldValue);
            }

            if ($add === false) {
                break;
            }
        }


        if ($add === true) {
            return array('error' => '0', 'ruleId' => $ruleId);
        }

        return array('error' => '1', 'ruleId' => $ruleId);
    }


    public static function filterDomain($conversationPage)
    {
        $fullDomain = 'http://' . strtr($conversationPage, array('http://' => '', 'https://' => ''));
        $domain = parse_url($fullDomain, PHP_URL_HOST) . parse_url($fullDomain, PHP_URL_PATH);
        if (empty($domain)) {
            return '';
        }
        $domain  = rtrim($domain, '/');
        $domain = $domain.'/';
        $domain = substr($domain, 0, strrpos($domain, '/'));

        return $domain;
    }


    /**
     * Check whether value meet criteria
     *
     * @param string $comparisonOperator comparison operator ('equal','notequal','like','in','greaterthan','lessthan', 'notin')
     * @param string $currentValue lead value
     * @param string $compareTo config value
     * @return bool
     */
    public static function compareValue($comparisonOperator, $currentValue, $compareTo)
    {
        $skipStatus = false;
        $compareTo = mb_strtolower($compareTo);
        if (is_array($currentValue)) {
            $dataValue = array_map('mb_strtolower', $currentValue);
        } else {
            $dataValue = mb_strtolower($currentValue);
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_OPERATOR_EQUAL) {
            if (is_array($dataValue)) {
                if (!in_array($compareTo, $dataValue)) {
                    $skipStatus = true;
                }
            } else {
                if ($dataValue !== $compareTo) {
                    $skipStatus = true;
                }
            }

        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_OPERATOR_NOTEQUAL) {
            if (is_array($dataValue)) {
                if (in_array($compareTo, $dataValue)) {
                    $skipStatus = true;
                }
            } else {
                if ($dataValue === $compareTo) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_OPERATOR_LIKE) {
            if (is_array($dataValue)) {
                $inputSearchPattern = preg_quote($compareTo, '~');
                $inputSearchResult = preg_grep('~' . $inputSearchPattern . '~', $dataValue);
                if (empty($inputSearchResult)) {
                    $skipStatus = true;
                }
            } else {
                $matchFound = mb_strpos($dataValue, $compareTo);
                if ($matchFound === false) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_OPERATOR_IN) {
            $compareToArray = explode(',', $compareTo);
            $compareToArray = array_map("mb_strtolower", $compareToArray);
            if (is_array($dataValue)) {
                $comparisonResult = array_intersect($compareToArray, $dataValue);
                if (count($comparisonResult) == 0) {
                    $skipStatus = true;
                }

            } else {
                if (!in_array($dataValue, $compareToArray)) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_NOT_IN) {
            $compareToArray = explode(',', $compareTo);
            $compareToArray = array_map("mb_strtolower", $compareToArray);
            if (is_array($dataValue)) {
                $comparisonResult = array_intersect($compareToArray, $dataValue);
                if (count($comparisonResult) > 0) {
                    $skipStatus = true;
                }

            } else {
                if (in_array($dataValue, $compareToArray)) {
                    $skipStatus = true;
                }
            }
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_GREATER_THAN) {
            if (is_numeric($dataValue)) {
                if ($compareTo > $dataValue) {
                    $skipStatus = true;
                }
            } elseif (is_array($dataValue)) {
                foreach ($dataValue as $value) {
                    if (is_numeric($value)) {
                        if ($compareTo > $dataValue) {
                            $skipStatus = true;
                        }
                    } else {
                        $skipStatus = true;
                    }
                }
            } else {
                $skipStatus = true;
            }
        }

        if ($comparisonOperator === Olarkchat_Models_Models_OlarkchatRulesConfigModel::RULE_COMPARISON_LESS_THAN) {
            if (is_numeric($dataValue)) {
                if ($compareTo < $dataValue) {
                    $skipStatus = true;
                }
            } elseif (is_array($dataValue)) {
                foreach ($dataValue as $value) {
                    if (is_numeric($value)) {
                        if ($compareTo < $dataValue) {
                            $skipStatus = true;
                        }
                    } else {
                        $skipStatus = true;
                    }
                }
            } else {
                $skipStatus = true;
            }
        }

        if ($skipStatus === false) {
            return true;
        }

        return false;

    }

}