import {mapGetters} from 'vuex';
import pagination from '../../pagination';
import moment from 'moment';
export default {
    data () {
        return {
            loadedForm: false,
            loadedGrid: false,
            websiteUrl: $('#website_url').val(),
            properties: {
                'domain': "Domain name",
                'url': "Url"
            },
            chosenProperty: '0',
            propertyDataEl: [],
            selectedOwner: '0',
            ruleName: '',
            actionsData: [],
            operators:{
                'domain' :
                    {
                        'equal': "Equal",
                        'like' : "Like"
                    },
                'url' :   {
                    'equal': "Equal",
                    'like' : "Like"
                },
            },
            placeholders: {
                'domain': 'www.domain.com',
                'url': 'contact.html'
            }
        }
    },
    components: {
        pagination: pagination
    },
    computed: {
        configDataInfo: function() {
            return this.$store.getters.getConfigDataInfo;
        },
        configScreenInfo: function() {
            return this.$store.getters.getConfigScreenInfo;
        },
        leadOwners: function() {
           return this.alphabeticalSort(this.$store.getters.getConfigScreenInfo.leadOwners);
        }
    },
    methods: {
        alphabeticalSort: function(obj){
            // convert object into array
            var sortable=[];
            for(var key in obj)
                if(obj.hasOwnProperty(key))
                    sortable.push([key, obj[key]]); // each item is an array in format [key, value]

            // sort items by value
            sortable.sort(function(a, b)
            {
                var x=a[1].toLowerCase(),
                    y=b[1].toLowerCase();
                return x<y ? -1 : x>y ? 1 : 0;
            });
            return sortable; // array in format [ [ key1, val1 ], [ key2, val2 ], ... ]
        },
        addProperty: function(name)
        {
            if (name == "0") {
                return false;
            }

            let found = this.propertyDataEl.find(function(obj) {
                return obj.name == name;
            });

            if (typeof found === 'undefined') {
                this.propertyDataEl.push({
                    'name': name,
                    'operators': this.operators[name],
                    'value' : '',
                    'label' : this.properties[name],
                    'operator': 'equal',
                    'placeholder' : this.placeholders[name]
                })
            }
        },
        deletePropertyData: function(index)
        {
            this.propertyDataEl.splice(index,1)
            this.chosenProperty = '0';
        },
        prepareDate: function(createdAt) {
            if (moment(createdAt, 'YYYY-MM-DD HH:mm:ss').format('DD MMMM YYYY HH:mm:ss') !== 'Invalid date') {
                return moment(createdAt, 'YYYY-MM-DD HH:mm:ss').format('DD')  + ' ' + moment(createdAt, 'YYYY-MM-DD HH:mm:ss').format('MMM') + ' ' + moment(createdAt, 'YYYY-MM-DD HH:mm:ss').format('YYYY');
            }
            return '';
        },
        resetForm: function()
        {
            this.actionsData = [];
            this.chosenProperty = '0';
            this.selectedOwner = 0;
            this.ruleName = '';
            this.propertyDataEl = [];
        },
        async addRule(e){
            if (this.chosenProperty == '0') {
                showMessage('Please specify at least one property/action', true, 2000);
                return false;
            }

            if (this.selectedOwner == '0') {
                showMessage('Please specify lead owner', true, 2000);
                return false;
            }

            if (this.ruleName == '') {
                showMessage('Please specify rule name', true, 2000);
                return false;
            }

            this.actionsData.push({
                'actionType': 'assign_owner',
                'assign_to' : this.selectedOwner
            });

            const result = await this.$store.dispatch('saveConfigData', {
                'ruleName':this.ruleName,
                'fieldsData':this.propertyDataEl,
                'actionsData': this.actionsData
            });

            if (result.status === 'error') {
                showMessage(result.message, true, 2000);
                return false;
            } else {
                this.resetForm();
                showMessage(result.message, false, 2000);
                const resultConfigData = await this.$store.dispatch('getConfigSavedData', {'router':this.$router});
                if(result.status === 'error') {

                } else {
                    this.loadedGrid = true;
                }
            }
        },
        goToRuleDetailScreen: function (ruleId)
        {
            this.$router.push({ name: 'rulesdetails', params: {'id': ruleId}});
        },
        async deleteConfigItem(id){
            showConfirm('Are you sure?', async () => {
                const result = await this.$store.dispatch('deleteConfigRecord', {'id': id});
                console.log('deleteTab',result);
                if (result.status === 'error') {
                    showMessage(result.message, true, 2000);
                    return false;
                } else {
                    showMessage(result.message, false, 2000);
                    const resultConfigData = await this.$store.dispatch('getConfigSavedData', {'router':this.$router});
                    if(result.status === 'error') {

                    } else {
                        this.loadedGrid = true;
                    }
                }
            });
        }
    },
    async created(){
        const result = await this.$store.dispatch('getConfigSavedData', {'router':this.$router});
        console.log('created', result);
        if(result.status === 'error') {
            showMessage('Please re-login', true, 3000);
        } else {
            this.loadedForm = true;
        }
        if(result.status === 'error') {

        } else {
            this.loadedGrid = true;
        }
    },
    async updated() {
        this.$nextTick(function () {
            if (typeof checkboxRadioStyle !== 'undefined' && typeof checkboxRadioStyle() === "function") {
                checkboxRadioStyle();
            }
        })
    }
}